#!/usr/bin/env python3

import argparse
import os
import queue
import sounddevice as sd
import vosk
import sys
import json
import string
import re
import random
import knowledge_base as kb 

class Eliza(object):

	def __init__(self):
		self.keys = list(map(lambda x: re.compile(x[0], re.IGNORECASE), gPats))
		self.values = list(map(lambda x: x[1], gPats))

	def translate(self, str, dict):
		words = str.lower().split()
		keys = dict.keys()
		for i in range(len(words)):
			if words[i] in keys:
				words[i] = dict[words[i]]
		return " ".join(words)

	def respond(self, str):
		for i in range(50):
			match = self.keys[i].match(str)
			if match:
				resp = random.choice(self.values[i])
				pos  = resp.find('%')
				while pos > -1:
					num = int(resp[pos+1 : pos+2])
					resp = resp[:pos] + self.translate(match.group(num), gReflections) + resp[pos+2 :]
					pos = resp.find('%')
				if resp[-2:] == '?.': resp = resp[:-2] + '.'
				if resp[-2:] == '??': resp = resp[:-2] + '?'
				return resp

gReflections = kb.verbs
gPats = kb.queries

q = queue.Queue()

def int_or_str(text):
    """Helper function for argument parsing."""
    try:
        return int(text)
    except ValueError:
        return text

def callback(indata, frames, time, status):
    """This is called (from a separate thread) for each audio block."""
    if status:
        print(status, file=sys.stderr)
    q.put(bytes(indata))

parser = argparse.ArgumentParser(add_help=False)
parser.add_argument(
    '-l', '--list-devices', action='store_true',
    help='show list of audio devices and exit')
args, remaining = parser.parse_known_args()
if args.list_devices:
    print(sd.query_devices())
    parser.exit(0)
parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=argparse.RawDescriptionHelpFormatter,
    parents=[parser])
parser.add_argument(
    '-f', '--filename', type=str, metavar='FILENAME',
    help='audio file to store recording to')
parser.add_argument(
    '-m', '--model', type=str, metavar='MODEL_PATH',
    help='Path to the model')
parser.add_argument(
    '-d', '--device', type=int_or_str,
    help='input device (numeric ID or substring)')
parser.add_argument(
    '-r', '--samplerate', type=int, help='sampling rate')
args = parser.parse_args(remaining)
terapista = Eliza()
try:
    if args.model is None:
        args.model = "model"
    if not os.path.exists(args.model):
        print ("Please download a model for your language from https://alphacephei.com/vosk/models")
        print ("and unpack as 'model' in the current folder.")
        parser.exit(0)
    if args.samplerate is None:
        device_info = sd.query_devices(args.device, 'input')
        # soundfile expects an int, sounddevice provides a float:
        args.samplerate = int(device_info['default_samplerate'])

    model = vosk.Model(args.model)

    if args.filename:
        dump_fn = open(args.filename, "wb")
    else:
        dump_fn = None

    with sd.RawInputStream(samplerate=args.samplerate, blocksize = 8000, device=args.device, dtype='int16',
                            channels=1, callback=callback):
            rec = vosk.KaldiRecognizer(model, args.samplerate)
            while True:
                data = q.get()
                if rec.AcceptWaveform(data):
                    res = json.loads(rec.Result())
                    if res['text'] == "":
                        continue
                    print (terapista.respond(str(res['text'])))
                    sys.stdout.flush()
                if dump_fn is not None:
                    continue

except KeyboardInterrupt:
    print('\nDone')
    parser.exit(0)
except Exception as e:
    parser.exit(type(e).__name__ + ': ' + str(e))
